---------------
-- LIBRARIES --
---------------
local AceAddon = LibStub("AceAddon-3.0");
local L = LibStub("AceLocale-3.0"):GetLocale("Briefings");
Briefings = AceAddon:NewAddon("Briefings", "AceConsole-3.0", "AceEvent-3.0");

DefaultDBSettings = {
	global = {
		settingsCommand = "briefings",
		settingsPanel = "settings",
		briefingChannel = "RAID",
		briefingCommand = "brief",
		topics = {
	
		}
	}	
}
----------
-- CORE --
----------
function Briefings:OnInitialize()
	Briefings.db = LibStub("AceDB-3.0"):New("BriefingsDB", DefaultDBSettings, "global")
	Briefings.apiVersion = 2;
	Briefings:RegisterChatCommand(Briefings.db.global.settingsCommand, function() OpenOptionsWindow() end);
	Briefings:RegisterChatCommand(Briefings.db.global.briefingCommand, function(msg) OutputBriefing(msg) end)
	Briefings:InitializeTopics();
end

function OpenOptionsWindow()
	if (Briefings.db.global.settingsPanel == "settings") then
		InterfaceOptionsFrame_OpenToCategory(Briefings.optionsFrame);
		InterfaceOptionsFrame_OpenToCategory(Briefings.optionsFrame);
	else
		InterfaceOptionsFrame_OpenToCategory(Briefings.topicsFrame);
		InterfaceOptionsFrame_OpenToCategory(Briefings.topicsFrame);
	end
end

function OutputBriefing(topic)
	if (not topic or topic == "") then
		print("|cffff0000[Briefings]|r "..L["Please specify a topic to output."]);
		return;
	end
	topic = topic:lower();
	if (not Briefings.db.global.topics[topic]) then
		print("|cffff0000[Briefings]|r "..L["Topic"].." [|cffff0000"..topic.."|r] "..L["could not be found. Are you sure you have that topic configured?"]);
		return;
	end
	local channel = ValidateChannel(Briefings.db.global.briefingChannel);
	if (not channel) then return end;
	local topicContents = Briefings.db.global.topics[topic];
	topicContents = topicContents:gsub("[\t]+"," ");
	local topicLines = split(topicContents, "\n");
	local i = 1;
	local ticker = C_Timer.NewTicker(0.5,function()
		SendChatMessage(topicLines[i],channel);
		i = i+1;
	end,#topicLines)
end

function ValidateChannel(channel) 
	if channel == "RAID" or channel == "PARTY" then
		if (IsInRaid() and channel == "RAID") then
			return "RAID";
		elseif (IsInGroup()) then
			return "PARTY";
		else
			print("|cffff0000[Briefings]|r "..L["Cannot send to Raid"])
			return nil;
		end
	elseif channel == "GUILD" then
		if (IsInGuild()) then
			return channel;
		else
			print("|cffff0000[Briefings]|r "..L["Cannot send to Guild"])
			return nil;
		end
	elseif channel == "BATTLEGROUND" then
		if (UnitInBattleground("player") ~= nil) then
			return channel;
		else
			print("|cffff0000[Briefings]|r "..L["Cannot send to Battleground"])
			return nil;
		end
	elseif channel == "INSTANCE_CHAT" then
		if (IsInInstance()) then
			return channel;
		else
			print("|cffff0000[Briefings]|r "..L["Cannot send to Instance"])
			return nil;
		end
	else
		return channel;
	end
end
function split(str, delim)
    local result,pat,lastPos = {},"(.-)" .. delim .. "()",1
    for part, pos in string.gmatch(str, pat) do
        table.insert(result, part); lastPos = pos
    end
    table.insert(result, string.sub(str, lastPos))
    return result
end
