local L = LibStub("AceLocale-3.0"):GetLocale("Briefings");

StaticPopupDialogs["CONFIRM_COMMAND_RELOAD"] = {
  text = L["CONFIRM_COMMAND_RELOAD"],
  button1 = L["Yes"],
  button2 = L["No"],
  OnAccept = function()
      ReloadUI();
  end,
  timeout = 0,
  whileDead = true,
  hideOnEscape = true,
  preferredIndex = 3,
}

Briefings.TopicObject = {
	type = "group",
	name = "",
	cmdHidden = true,
	args = {
		deleteButton = {
			type = "execute",
			name = L["Delete Topic"],
			desc = L["Delete the currently selected Topic"],
			confirm = function() return L["DELETE_CONFIRMATION"] end,
			func = function(info) 
				Briefings.db.global.topics[info[1]] = nil;
				info.options.args[info[1]] = nil;
			end,
			order = 1
		},
		contents = {
			type = "input",
			name = L["Contents"],
			desc = L["Contents_Desc"];
			width = "full",
			order = 2,
			multiline = 24,
			set = function(info,val) Briefings.db.global.topics[info[1]] = val end,
			get = function(info) return Briefings.db.global.topics[info[1]] end
		}
	}
}

Briefings.optionsFrameData = {
	type = "group",
	name = L["OptionsFrameTitle"],
	args = {
		settingsRow = {
			type = "group",
			name = "",
			cmdHidden = true,
            inline = true,
			order = 1,
			args = {
				settingsCommandInput = {
				name = L["Settings Command"],
				desc = L["Settings Command Desc"],
				type = "input",
				set = function(_,val)Briefings.db.global.settingsCommand = val; StaticPopup_Show("CONFIRM_COMMAND_RELOAD"); end,
				get = function(self)return Briefings.db.global.settingsCommand end,
				order = 1
				},
				settingsCommandPanel = {
					name = L["Panel to open"],
					desc = L["Pannel that should be opened with the settings command"],
					type = "select",
					values = {settings = "Briefings "..L["Settings"], topics = "Briefings "..L["Topics"]},
					set = function(info,val) Briefings.db.global.settingsPanel = val end,
					get = function() return Briefings.db.global.settingsPanel end
				}
			}
		},
		briefingRow = {
			type = "group",
			name = "",
			cmdHidden = true,
            inline = true,
			order = 2,
			args = {
				briefingCommandInput = {
				name = L["Briefing Command"],
				desc = L["Briefing Command Desc"],
				type = "input",
				set = function(_,val) Briefings.db.global.briefingCommand = val; StaticPopup_Show("CONFIRM_COMMAND_RELOAD"); end,
				get = function(self)
					local command = Briefings.db.global.briefingCommand;
					self.options.args.briefingRow.args.briefingCommandLabel.name = "/"..command.." "..L["<topic name>"];
					return command;
					end,
				order = 2
				},
				briefingChannel = {
					name = L["Briefing Channel"],
					desc = L["The channel that briefings should be sent to."],
					type = "select",
					values = {RAID = L["RAID"], PARTY = L["PARTY"], GUILD = L["GUILD"], BATTLEGROUND = L["BATTLEGROUND"], INSTANCE_CHAT = L["INSTANCE_CHAT"], YELL = L["YELL"], SAY = L["SAY"]},
					set = function(info,val) Briefings.db.global.briefingChannel = val end,
					get = function() return Briefings.db.global.briefingChannel end,
					order = 3
				},
				briefingCommandLabel = {
					name = "/brief <topic name>",
					type = "description",
					order = 4
				}
			}
		},
		resetRow = {
			type = "group",
			name = "",
			cmdHidden = true,
			inline = true,
			order = 3,
			args = {
				resetButton = {
					type = "execute",
					width = "double",
					name = L["Reset Briefings' Database"],
					desc = L["Reset Briefings to factory settings."],
					confirm = function() return L["Are you REALLY sure you want to reset Briefings to the factory settings? This will cause a reload."] end,
					func = function() 
					Briefings.db.global.topics = DefaultDBSettings.global.topics;
					Briefings.db.global.settingsCommand = DefaultDBSettings.global.settingsCommand;
					Briefings.db.global.settingsPanel = DefaultDBSettings.global.settingsPanel;
					Briefings.db.global.briefingChannel = DefaultDBSettings.global.briefingChannel;
					Briefings.db.global.briefingCommand = DefaultDBSettings.global.briefingCommand;
					ReloadUI();
					end
				}
			}
		}
	}
};
Briefings.topicFrameData = {
	name = L["Briefing Topics"],
	type = "group",
	args = {
		ManagementRow = {
			type = "group",
			name = "",
			inline = true,
			cmdHidden = true,
			order = 1,
			args = {
				createInput = {
					type = "input",
					name = L["Create new Topic"],
					get = function() return "" end,
					set = function(info,val)
						val = val:lower();
						if (Briefings.db.global.topics[val]) then return end;
						Briefings.db.global.topics[val] = "";
						info.options.args[val] = deepcopy(Briefings.TopicObject);
						info.options.args[val].name = val;
					end,
					order = 1
				},
				ImportantNote = {
					type = "description",
					name = L["Briefing_Content_Note"];
				}
			}
		}
	}
}

function Briefings:InitializeTopics()
	for k,v in pairs(Briefings.db.global.topics) do
		local topic = deepcopy(Briefings.TopicObject);
		topic.name = k;
		Briefings.topicFrameData.args[k] = topic;
	end
	AceConfigRegistry:RegisterOptionsTable("Briefing Topics",Briefings.topicFrameData);
	Briefings.topicsFrame = AceConfigDialog:AddToBlizOptions(L["Briefing Topics"], "Briefing Topics", "Briefings");
end

function deepcopy(orig)
    local orig_type = type(orig)
    local copy
    if orig_type == 'table' then
        copy = {}
        for orig_key, orig_value in next, orig, nil do
            copy[deepcopy(orig_key)] = deepcopy(orig_value)
        end
        setmetatable(copy, deepcopy(getmetatable(orig)))
    else -- number, string, boolean, etc
        copy = orig
    end
    return copy
end

AceConfigDialog = LibStub("AceConfigDialog-3.0");
AceConfigRegistry = LibStub("AceConfigRegistry-3.0");
AceConfigRegistry:RegisterOptionsTable("Briefings",Briefings.optionsFrameData);
Briefings.optionsFrame = AceConfigDialog:AddToBlizOptions("Briefings", "Briefings");